// (C) Denisov A.W., 2003
// mailto:alwd@narod.ru
// http://alwd.narod.ru

{$I-}

unit LogFile;

interface

//        LOG-
//   FileName := ChangeFileExt(Application.ExeName,'.log');
// .  

var
  FileName: string = '';

function IntToBinaryStr(Value: integer; BitsCount: byte): string;
function IntToHexStr(Value: integer; TetraCount: byte): string;

//    LOG-:
// < Comment + ': ' + Value >

//        .
//       TRUE,  FALSE.

//  LogTruncateFile      ,
//     .

function LogWriteString(const LogStr: string): boolean;
function LogWriteStringWithComment(const Comment, Str: string): boolean;
function LogWriteInteger(const Comment: string; Value: integer): boolean;
function LogWriteFloat(const Comment: string; Value: extended): boolean;
//  BitsCount    ,   LOG,
//     1  32
function LogWriteIntegerAsBinaryStr(const Comment: string; Value: integer; BitsCount: byte): boolean;
//  TetraCount    ,   LOG,
//     1  8
function LogWriteIntegerAsHexStr(const Comment: string; Value: integer; TetraCount: byte): boolean;

function LogDeleteFile: boolean;
function LogTruncateFile(MaxSize: integer): boolean;
function LogFileSize: integer;

implementation

uses
  SysUtils, Forms, Classes;


const
  Colon = ': ';


function HexChar(b: byte): char;
begin
  if b < 10 then Result := Chr(b + Ord('0'))
  else Result := Chr(b - 10 + Ord('A'));
end;


function IntToBinaryStr(Value: integer; BitsCount: byte): string;
var
  i: integer;
begin
  SetLength(Result,BitsCount);
  for i := BitsCount downto 1 do begin
    if (Value and 1) = 0 then Result[i] := '0' else Result[i] := '1';
    Value := Value shr 1;
  end;
end;


function IntToHexStr(Value: integer; TetraCount: byte): string;
var
  i: integer;
begin
  SetLength(Result,TetraCount);
  for i := TetraCount downto 1 do begin
    Result[i] := HexChar(byte(Value) and $0F);
    Value := Value shr 4;
  end;
end;


function LogWriteString(const LogStr: string): boolean;
var
  Log: text;
begin
  Result := TRUE;
  if (Length(LogStr) < 1) or (Length(FileName) < 1) then Exit;
  AssignFile(Log,FileName);
  if FileExists(FileName) then Append(Log) else Rewrite(Log);
  try
    WriteLn(Log,LogStr);
  except
    Result := FALSE;
  end;
  try
    Close(Log);
  except
  end;
end;


function LogWriteStringWithComment(const Comment, Str: string): boolean;
begin
  Result := LogWriteString(Comment + Colon + Str);
end;


function LogWriteInteger(const Comment: string; Value: integer): boolean;
begin
  Result := LogWriteString(Comment + Colon + IntToStr(Value));
end;


function LogWriteFloat(const Comment: string; Value: extended): boolean;
begin
  Result := LogWriteString(Comment + Colon + FloatToStr(Value));
end;


function LogWriteIntegerAsBinaryStr(const Comment: string; Value: integer; BitsCount: byte): boolean;
begin
  Result := TRUE;
  if BitsCount < 1 then Exit;
  if BitsCount > 32 then BitsCount := 32;
  Result := LogWriteString(Comment + Colon + IntToBinaryStr(Value,BitsCount));
end;


function LogWriteIntegerAsHexStr(const Comment: string; Value: integer; TetraCount: byte): boolean;
begin
  Result := TRUE;
  if TetraCount < 1 then Exit;
  if TetraCount > 8 then TetraCount := 8;
  Result := LogWriteString(Comment + Colon + IntToHexStr(Value,TetraCount));
end;


function LogDeleteFile;
begin
  Result := TRUE;
  try
    if FileExists(FileName) then DeleteFile(FileName);
  except
    Result := FALSE;
  end;
end;


//    ,  ,  ,
//  ,     StringList,   
//   ,  .
function LogTruncateFile(MaxSize: integer): boolean;
var
  Delta: integer;
  f: text;
  i: integer;
  SL: TStringList;
  s: string;
  DelCount: integer;
begin
  Result := TRUE;
  Delta := LogFileSize - MaxSize;
  if Delta <= 0 then Exit;
  if FileExists(FileName) then begin
    SL := TStringList.Create;
    AssignFile(f,FileName);
    try
      Reset(f);
      DelCount := 0;
      //      NULL,   
      //   .
      repeat
        ReadLn(f,s);
        DelCount := DelCount + Length(s);
      until (DelCount >= Delta) or EOF(f);
      //  -  StringList
      while not EOF(f) do begin
        ReadLn(f,s);
        SL.Add(s);
      end;
    finally
      Close(f);
    end;
    if SL.Count > 0 then begin
      //     .
      try
        Rewrite(f);
        for i := 0 to SL.Count - 1 do begin
          WriteLn(f,SL[i]);
        end;
      finally
        Close(f);
      end;
    end;
    SL.Free;
  end;
end;


function LogFileSize: integer;
var
  f: file;
begin
  Result := 0;
  if Length(FileName) < 1 then Exit;
  if FileExists(FileName) then begin
    try
      AssignFile(f,FileName);
      Reset(f,1);
      Result := FileSize(f);
    except
    end;
    try
      CloseFile(f);
    except
    end;
  end;
end;


initialization

  FileName := ChangeFileExt(Application.ExeName,'.log');

end.
